/**********************************************************************************************
 Copyright (c) 2014 DisplayModule. All rights reserved.

 Redistribution and use of this source code, part of this source code or any compiled binary
 based on this source code is permitted as long as the above copyright notice and following
 disclaimer is retained.

 DISCLAIMER:
 THIS SOFTWARE IS SUPPLIED "AS IS" WITHOUT ANY WARRANTIES AND SUPPORT. DISPLAYMODULE ASSUMES
 NO RESPONSIBILITY OR LIABILITY FOR THE USE OF THE SOFTWARE.
 ********************************************************************************************/

 //Tested on NUCLEO-F401RE, LPCXpresso11U68 platform.

#include "DmTftS6D0164.h"



DmTftS6D0164::DmTftS6D0164(PinName rd, PinName wr, PinName dc, PinName cs , PinName rst) : DmTftBase(240, 320) 
{
  _rd = rd;
  _wr = wr;
  _cs = cs;
  _dc = dc;
  _rst = rst;
}

DmTftS6D0164::~DmTftS6D0164() {
#if defined (DM_TOOLCHAIN_MBED)
  delete _pinRST;
  delete _pinCS;
  delete _pinWR;
  delete _pinDC;
  delete _virtualPortD;
  _pinRST = NULL;
  _pinCS = NULL;
  _pinWR = NULL;
  _pinDC = NULL;
  _virtualPortD = NULL;
#endif
}

void DmTftS6D0164::writeBus(uint8_t data) {
#if defined (DM_TOOLCHAIN_ARDUINO)
  PORTD = data;
#elif defined (DM_TOOLCHAIN_MBED)
  *_virtualPortD = data;
#endif
  pulse_low(_pinWR, _bitmaskWR);
}

void DmTftS6D0164::sendCommand(uint8_t index) {
  cbi(_pinDC, _bitmaskDC);
  writeBus(index);
}

void DmTftS6D0164::sendData(uint16_t data) {
  sbi(_pinDC, _bitmaskDC);
  writeBus(data>>8);
  writeBus(data);
}

void DmTftS6D0164::send8BitData(uint8_t data)
{
  sbi(_pinDC, _bitmaskDC);
  writeBus(data);
}

void DmTftS6D0164::setAddress(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1) {
  // sendCommand(0x37); // Set Column
  // sendData(x0);
  // sendCommand(0x36);
  // sendData(x1);


  // sendCommand(0x39);  // Set Page
  // sendData(y0);
  // sendCommand(0x38);
  // sendData(y1);

  // sendCommand(0x20);
  // sendData(x0);
  // sendCommand(0x21);
  // sendData(y0);
  // sendCommand(0x22);
   sendCommand(0x2A); // Set Column
  sendData(x0);
  sendData(x1);

  sendCommand(0x2B);  // Set Page
  sendData(y0);
  sendData(y1);

  sendCommand(0x2c);
  
}

void DmTftS6D0164::init(void) {
  setTextColor(BLACK, WHITE);
#if defined (DM_TOOLCHAIN_ARDUINO)
#elif defined (DM_TOOLCHAIN_MBED)
  _pinRST = new DigitalOut(_rst);
  _pinCS = new DigitalOut(_cs);
  _pinWR = new DigitalOut(_wr);
  _pinDC = new DigitalOut(_dc);
  _pinRD = new DigitalOut(_rd);
  _virtualPortD = new BusOut(D8, D9, D2, D3, D4, D5, D6, D7);
#endif
  sbi(_pinRD,_bitmaskRST);
  delay(5);
  sbi(_pinRST, _bitmaskRST);
  delay(5);
  cbi(_pinRST, _bitmaskRST);
  delay(15);
  sbi(_pinRST, _bitmaskRST);
  sbi(_pinCS, _bitmaskCS);
  sbi(_pinWR, _bitmaskWR);
  delay(15);
  cbi(_pinCS, _bitmaskCS);

  // Power up sequence
  // ILI9341 init
  // while(1)
  // {
  //   printf("data is execued \r\n");
  //   sendCommand(0x11);
  //   //delay(120);
  //   wait_ms(1);
  // }

  sendCommand(0xCF);  
  send8BitData(0x00); 
  send8BitData(0xC1); 
  send8BitData(0X30); 
  sendCommand(0xED);  
  send8BitData(0x64); 
  send8BitData(0x03); 
  send8BitData(0X12); 
  send8BitData(0X81); 
  sendCommand(0xE8);  
  send8BitData(0x85); 
  send8BitData(0x10); 
  send8BitData(0x7A); 
  sendCommand(0xCB);  
  send8BitData(0x39); 
  send8BitData(0x2C); 
  send8BitData(0x00); 
  send8BitData(0x34); 
  send8BitData(0x02); 
  sendCommand(0xF7);  
  send8BitData(0x20); 
  sendCommand(0xEA);  
  send8BitData(0x00); 
  send8BitData(0x00); 
  sendCommand(0xC0);    //Power control 
  send8BitData(0x1B);   //VRH[5:0] 
  sendCommand(0xC1);    //Power control 
  send8BitData(0x01);   //SAP[2:0];BT[3:0] 
  sendCommand(0xC5);    //VCM control 
  send8BitData(0x30);   //3F
  send8BitData(0x30);   //3C
  sendCommand(0xC7);    //VCM control2 
  send8BitData(0XB7); 
  sendCommand(0x36);    // Memory Access Control 
  send8BitData(0x48); 
  sendCommand(0x3A);   
  send8BitData(0x55); 
  sendCommand(0xB1);   
  send8BitData(0x00);   
  send8BitData(0x1A); 
  sendCommand(0xB6);    // Display Function Control 
  send8BitData(0x0A); 
  send8BitData(0xA2); 
  sendCommand(0xF2);    // 3Gamma Function Disable 
  send8BitData(0x00); 
  sendCommand(0x26);    //Gamma curve selected 
  send8BitData(0x01); 
  sendCommand(0xE0);    //Set Gamma 
  send8BitData(0x0F); 
  send8BitData(0x2A); 
  send8BitData(0x28); 
  send8BitData(0x08); 
  send8BitData(0x0E); 
  send8BitData(0x08); 
  send8BitData(0x54); 
  send8BitData(0XA9); 
  send8BitData(0x43); 
  send8BitData(0x0A); 
  send8BitData(0x0F); 
  send8BitData(0x00); 
  send8BitData(0x00); 
  send8BitData(0x00); 
  send8BitData(0x00);     
  sendCommand(0XE1);    //Set Gamma 
  send8BitData(0x00); 
  send8BitData(0x15); 
  send8BitData(0x17); 
  send8BitData(0x07); 
  send8BitData(0x11); 
  send8BitData(0x06); 
  send8BitData(0x2B); 
  send8BitData(0x56); 
  send8BitData(0x3C); 
  send8BitData(0x05); 
  send8BitData(0x10); 
  send8BitData(0x0F); 
  send8BitData(0x3F); 
  send8BitData(0x3F); 
  send8BitData(0x0F); 
  sendCommand(0x2B); 
  send8BitData(0x00);
  send8BitData(0x00);
  send8BitData(0x01);
  send8BitData(0x3f);
  sendCommand(0x2A); 
  send8BitData(0x00);
  send8BitData(0x00);
  send8BitData(0x00);
  send8BitData(0xef); 

  sendCommand(0x11); //Exit Sleep
  delay(120);
  sendCommand(0x29); //display on 
  sbi(_pinCS, _bitmaskCS);
  clearScreen(RED);
}

/*********************************************************************************************************
  END FILE
*********************************************************************************************************/


